ODA({
    is: 'oda-jupyter', imports: '@oda/button',
    template: `
        <style>
            :host{
                @apply --vertical;
                @apply --flex;
                padding: 12px 6px;
                height: calc(100% - 24px);
                opacity: 0;
                transition: opacity ease-in .5s;
            }
        </style>
        <oda-jupyter-divider idx="-1" :focused="!notebook?.cells?.length"></oda-jupyter-divider>
        <div ~for="notebook?.cells" class="vertical no-flex">
            <div ~is="editors?.[$for.item.cell_type].editor" :idx="$for.index" :cell="$for.item" :shadow="focusedIdx === $for.index" :focused="focusedIdx === $for.index" @tap.stop="focusedIdx = (_readOnly ? -1 : $for.index);"></div>
            <oda-jupyter-divider :idx="$for.index" style="margin-top: 4px;"></oda-jupyter-divider>
        </div>
    `,
    $public: {
        $pdp: true,
        iconSize: 24,
        readOnly: false
    },
    $pdp: {
        notebook: null,
        editors: {
            code: { label: 'Код', editor: 'oda-jupyter-code-editor' },
            text: { label: 'Текст', editor: 'oda-jupyter-text-editor' }
        },
        get arrEditors() {
            const ed = this.editors;
            return Object.keys(ed).map(k => { return { ...{ type: k }, ...ed[k] } });
        },
        focusedIdx: {
            $def: -1,
            set(n) {
                this.editIdx = -1;
            }
        },
        editIdx: -1,
        get _readOnly() {
            return this.notebook?.readOnly || this.readOnly;
        }
    },
    attached() {
        this.style.opacity = 1;
    },
    $listeners: {
        tap(e) { this.focusedIdx = this.editIdx = -1 }
    }
})

ODA({
    is: 'oda-jupyter-divider',
    template: `
        <style>
            :host {
                @apply --vertical;
                height: 8px;
                justify-content: center;
                z-index: 99;
                opacity: 0;
                transition: opacity ease-out .5s;
                position: relative;
                padding: 1px;
            }
            :host([focused]) {
                box-shadow: none !important;
            }
            :host([focused]) {
                opacity: 1;
            }
            :host(:hover) {
                opacity: 1;
            }
            oda-button {
                font-size: 14px;
                margin: 0px 4px;
                @apply --content;
                @apply --border;
                padding: 0px 4px 0px 0px;
                border-radius: 4px;
            }
        </style>
        <div class="horizontal center">
            <div ~if="!_readOnly && idx!==-1" style="width: 100%; position: absolute; top: 2px; height: 1px; border-bottom: 2px solid gray;"></div>
            <oda-button ~if="!_readOnly" :icon-size icon="icons:add" ~for="arrEditors" @tap.stop="addCell($for.item)">{{$for.item.label}}</oda-button>
        </div>
    `,
    idx: -2,
    addCell(i) {
        let idx = this.idx + 1;
        this.focusedIdx = this.editIdx = -1;
        this.notebook ||= {};
        this.notebook.cells ||= [];
        this.notebook.cells.splice(idx, 0, { cell_type: i.type, source: '' });
        this.async(() => this.focusedIdx = idx);
    }
})

ODA({
    is: 'oda-jupyter-cell',
    template: `
        <style>
            :host {
                @apply --no-flex;
                position: relative;
                padding: 1px;
                margin: 0 2px;
                transition: opacity ease-out 1s;
            }
        </style>
        <oda-jupyter-toolbar :idx ~if="!_readOnly && focused" ~style="{top: '-' + (iconSize - 4) + 'px'}"></oda-jupyter-toolbar>
    `,
    idx: -2,
    focused: false
})

ODA({
    is: 'oda-jupyter-toolbar',
    template: `
        <style>
            :host {
                @apply --horizontal;
                @apply --no-flex;
                @apply --content;
                @apply --shadow;
                position: absolute;
                top: 0;
                right: 8px;
                padding: 1px;
                border-radius: 4px;
                z-index: 100;
            }
        </style>
        <oda-button :disabled="focusedIdx === 0" :icon-size icon="icons:arrow-back:90" @tap="moveCell(-1)"></oda-button>
        <oda-button :disabled="focusedIdx >= notebook?.cells?.length - 1" :icon-size icon="icons:arrow-back:270" @tap="moveCell(1)"></oda-button>
        <span style="width: 8px"></span>
        <oda-button :icon-size icon="icons:delete" @tap="deleteCell"></oda-button>
        <span style="width: 8px"></span>
        <oda-button :icon-size :icon="editIdx===idx?'icons:close':'editor:mode-edit'" @tap="editIdx = editIdx===idx ? -1 : idx"> </oda-button>
    `,
    idx: -2,
    cell: null,
    iconSize: 20,
    moveCell(v) {
        this.editIdx = -1;
        let idx = this.focusedIdx;
        const cells = this.notebook.cells.splice(idx, 1);
        idx = idx + v;
        idx = idx < 0 ? 0 : idx > this.notebook.cells.length ? this.notebook.cells.length : idx;
        this.notebook.cells.splice(idx, 0, cells[0])
        this.async(() => {
            this.focusedIdx = idx;
        })
    },
    deleteCell() {
        if (window.confirm(`Do you really want delete current cell ?`)) {
            this.editIdx = -1;
            this.notebook.cells.splice(this.focusedIdx, 1);
        }
    }
})

ODA({
    is: 'oda-jupyter-text-editor', imports: '@oda/simplemde-editor,  @oda/md-viewer', extends: 'oda-jupyter-cell',
    template: `
        <style>
            oda-md-viewer::-webkit-scrollbar { width: 0px; height: 0px; }
            :host {
                @apply --horizontal;
                @apply --flex;
            }
            oda-md-viewer {
                opacity: {{opacity}};
                border: none;
                outline: none;
            }
        </style>
        <oda-simplemde-editor ~if="!readOnly && editIdx===idx" style="max-width: 50%; min-width: 50%; padding: 0px; margin: 0px;" @change="editorValueChanged"></oda-simplemde-editor>
        <oda-md-viewer tabindex=0 class="flex" :srcmd="src || _src" :pmargin="'0px'" @dblclick="changeEditMode" @keypress="keyPress"></oda-md-viewer>
    `,
    set cell(n) {
        this.src = n?.source || '';
    },
    idx: -2,
    src: '',
    _src: 'Чтобы изменить содержимое ячейки, дважды нажмите на нее (или выберите "Ввод")',
    get opacity() {
        return this.src ? 1 : .3;
    },
    editorValueChanged(e) {
        this.cell.source = this.src = e.detail.value;
        this.fire('change', this.cell);
        console.log(this.cell)
    },
    keyPress(e) {
        if (e.key === 'Enter')
            this.changeEditMode();
    },
    changeEditMode() {
        this.editIdx = this.editIdx === this.idx ? -1 : this.idx;
        this.async(() => {
            this.$('oda-simplemde-editor').value = this.src;
        })
    }
})

ODA({
    is: 'oda-jupyter-code-editor', imports: '@oda/ace-editor', extends: 'oda-jupyter-cell',
    template: `
        <style>
            :host {
                @apply --horizontal;
                @apply --flex;
                outline: 1px solid var(--border-color);
                position: relative;
            }
            oda-icon {
                padding: 4px;
                height: {{iconSize}};
            }
            #icon-close {
                margin-top: {{iconCloseTop}};
            }
            #icon-close:hover {
                fill: red;
            }
            iframe {
                width: 100%; 
                border: none;
                min-height: 36px;
                height: 36px;
                overflow: auto;
            }
        </style>
        <div class="vertical"  style="border-right: 1px solid var(--border-color); padding: 4px 0px">
            <oda-icon :icon-size="iconSize" :icon="iconRun" @pointerover="iconRunOver='av:play-circle-outline'" @tap="run" @pointerout="iconRunOver=''" style="cursor: pointer; position: sticky; top: 0" :fill="isRun ? 'green' : 'black'"></oda-icon>
            <oda-icon id="icon-close" ~if="isRun && iconCloseShow" :icon-size="iconSize" :icon="iconClose" @tap="isRun=false; runConsoleData = undefined;" style="cursor: pointer; position: sticky; top: 24px"></oda-icon>
        </div>
        <div class="vertical flex">
            <div style="display: none; padding: 2px; font-size: xx-small">{{cell?.mode + ' - ' + (cell?.isODA ? 'isODA' : 'noODA')}}</div>
            <oda-ace-editor :src :mode="cell?.mode || 'javascript'" class="flex" show-gutter="false" max-lines="Infinity" :read-only style="padding: 8px 0" @change="editorValueChanged"></oda-ace-editor>   
            <div id="splitter1" ~if="isRun && cell?.mode==='html'" ~style="{borderTop: isRun ? '1px solid var(--border-color)' : 'none'}"></div>
            <div id="result">
                <iframe ~if="isRun && cell?.mode==='html'" :srcdoc></iframe>
                <div id="splitter2" ~if="isRun && runConsoleData" ~style="{borderTop: isRun ? '1px solid var(--border-color)' : 'none'}"></div>
                <div ~if="isRun && runConsoleData" style="min-height: 36px; margin: 2px 0;">
                    <div ~for="runConsoleData" style="padding: 4px;" ~style="runConsoleStyle($for.item)">{{$for.item.str}}</div>
                </div>
            </div>
        </div>
    `,
    set cell(n) {
        this.src = n?.source || '';
        this.setCellMode(this.src);
    },
    src: '',
    srcdoc: '',
    get iconRun() {
        return this.isRun ? 'av:play-circle-outline' : this.iconRunOver || 'bootstrap:code-square';
    },
    iconRunOver: '',
    iconClose: 'eva:o-close-circle-outline',
    iconCloseTop: '',
    iconCloseShow: false,
    isRun: false,
    runConsoleData: undefined,
    runConsoleStyle(i) {
        let colors = { log: 'gray', info: 'blue', warn: 'orange', error: 'red' };
        return `color: ${colors[i.method]}`;
    },
    setCellMode(src = this.$('oda-ace-editor')?.value || '') {
        this.cell.mode = 'javascript';
        this.cell.isODA = false;
        let oda = src.match(/ODA\b[^(]*\([\s\S]*}\s*?\)/gm);
        if (oda?.length)
            this.cell.isODA = true;
        // let regx = src.match(/<d\b[^>]*>[\s\S]*?<\/script\b[^>]*>/gm);
        let arr = ['</script>', '</html>', '</body>', '</head>', '<link', '<!DOCTYPE html>', '<meta '];
        let regx = new RegExp(arr.join('|'));
        if (regx.test(src))
            this.cell.mode = 'html';
        if (this.cell.mode !== 'html' && !this.cell.isODA) {
            // regx = src.match(/<\b[^>]*>[\s\S]*?<\/\b[^>]*>/gmi);
            regx = src.match(/<\b[^>]*>[\s\S]*?/gm);
            this.cell.mode = src.match(regx)?.length ? 'html' : 'javascript';
        }
    },
    editorValueChanged(e) {
        this.iconCloseTop = undefined;
        this.cell.source = e.detail.value;
        this.fire('change', this.cell);
        this.setCellMode();
    },
    overrideСonsole(w = window) {
        let console = w.console;
        if (!console) return;
        this.runConsoleData = [];
        w.runConsoleData = undefined;
        w._runConsoleData = [];
        if (!w.useJConsole) {
            w.useJConsole = true;
            w.print = (e) => console.log(e);
            w.log = (e) => console.log(e);
            w.info = (e) => console.info(e);
            w.warn = (e) => console.warn(e);
            w.error = (e) => console.error(e);
            let intercept = (method) => {
                let original = console[method];
                console[method] = function () {
                    let message = arguments;
                    if (original.apply) {
                        original.apply(console, arguments);
                    } else {
                        message = Array.prototype.slice.apply(arguments).join(' ');
                        original(message);
                    }
                    if (w.runConsoleData) {
                        w.runConsoleData.push({ method, str: Array.prototype.slice.apply(message).join(' ') });
                    } else {
                        w._runConsoleData.push({ method, str: Array.prototype.slice.apply(message).join(' ') });
                    }
                }
            }
            ['log', 'info', 'warn', 'error'].forEach(i => intercept(i));
        };
        this.runConsoleData = [...w._runConsoleData];
        w.runConsoleData = this.runConsoleData;
    },
    run() {
        this.iconCloseShow = false;
        this.runConsoleData = undefined;
        this.iconClose = 'spinners:180-ring-with-bg';
        this.iconCloseTop = undefined;
        this.srcdoc = '';
        if (this.cell.mode === 'javascript') {
            this.overrideСonsole();
            this.isRun = true;

            // let fn = new Function('', `with (this) {${this.cell.source}}`)
            let fn = new Function(`try { ${this.cell.source} } catch (e) { console.error(e) }`);
            fn();

            let str = this.cell.source;
            str = str.split('\n').filter(i => i).at(-1);
            window[str] && console.log(window[str]);
        } else {
            this.isRun = true;
            this.async(() => {
                const iframe = this.$('iframe');
                this.srcdoc = `
    <style>
        html, body {
            margin: 0;
            padding: 0;
            position: relative;
            font-family: monospace;
            font-size: 18px;
        }
        * *, *:before, *:after {  
            box-sizing: border-box;
        }
    </style>
    <script async>
        window._runConsoleData = [];
        var overrideСonsole = () => {
            var console = window.console;
            if (!console) return;
            window.print = (e) => console.log(e);
            window.log = (e) => console.log(e);
            window.info = (e) => console.info(e);
            window.warn = (e) => console.warn(e);
            window.error = (e) => console.error(e);
            var intercept = (method) => {
                var original = console[method];
                console[method] = function() {
                    var message = arguments;
                    if (original.apply) {
                        // Do this for normal browsers
                        original.apply(console, arguments);
                    } else {
                        // Do this for IE
                        message = Array.prototype.slice.apply(arguments).join(' ');
                        original(message);
                    }
                    if (window.runConsoleData) {
                        window.runConsoleData.push({ method, str: Array.prototype.slice.apply(message).join(' ') });
                    } else {
                        window._runConsoleData.push({ method, str: Array.prototype.slice.apply(message).join(' ') });
                    }
                }
            }
            ['log', 'info', 'warn', 'error'].forEach(i => intercept(i));
        }
        overrideСonsole();
    </script>
    <script type="module">import '../../oda.js'</script>
                ` + this.cell.source;
                iframe.addEventListener('load', () => {
                    this.runConsoleData = [...iframe.contentWindow._runConsoleData];
                    iframe.contentWindow.runConsoleData = this.runConsoleData;
                    this.async(() => {
                        iframe.style.height = iframe.contentDocument.body.scrollHeight + 'px';
                        iframe.style.opacity = 1;
                    }, 300)
                })
            })
        }
        this.iconClose = 'eva:o-close-circle-outline';
        this.async(() => {
            this.iconCloseTop = (this.$('#splitter1')?.offsetTop || this.$('#splitter2')?.offsetTop || 36) - 36 + 'px';
            this.iconCloseShow = true;
            this.$('#result')?.scrollIntoView({ block: 'center', behavior: 'smooth' });
        }, 100)
    }
})
